@@ -4,13 +4,16 @@ require 'test_helper'
 require 'file/tail'
 require 'timeout'
 require 'thread'
+require 'fileutils'
 Thread.abort_on_exception = true
 
 class FileTailTest < Test::Unit::TestCase
   include File::Tail
+  include FileUtils
 
   def setup
-    @out = File.new("test.#$$", "wb")
+    @out = File.new(File.join(__dir__, "test.#$$"), "wb")
+    at_exit { rm_f File.expand_path(@out.path) }
     append(@out, 100)
     @in = File.new(@out.path, "rb")
     @in.extend(File::Tail)
@@ -194,7 +197,7 @@ class FileTailTest < Test::Unit::TestCase
     lines = []
     logger = Thread.new do
       begin
-        Timeout::timeout(2) do
+        Timeout::timeout(10) do
           @in.tail do |l|
             lines << l
           end
@@ -225,7 +228,7 @@ class FileTailTest < Test::Unit::TestCase
     lines = []
     logger = Thread.new do
       begin
-        Timeout::timeout(2) do
+        Timeout::timeout(10) do
           @in.tail do |l|
             lines << l
           end
@@ -261,7 +264,7 @@ class FileTailTest < Test::Unit::TestCase
     lines = []
     logger = Thread.new do
       begin
-        Timeout::timeout(2) do
+        Timeout::timeout(10) do
           @in.tail(15) do |l|
             lines << l
           end
@@ -298,7 +301,7 @@ class FileTailTest < Test::Unit::TestCase
     lines = []
     logger = Thread.new do
       begin
-        Timeout::timeout(2) do
+        Timeout::timeout(10) do
           @in.tail(110) do |l|
             lines << l
           end
@@ -307,8 +310,10 @@ class FileTailTest < Test::Unit::TestCase
       end
     end
     appender = Thread.new do
-      until logger.stop?
-        sleep 0.1
+      Timeout::timeout(10) do
+        until lines.size == 100
+          sleep 0.1
+        end
       end
       @out.close
       File.unlink(@out.path)
@@ -331,7 +336,7 @@ class FileTailTest < Test::Unit::TestCase
     lines = []
     logger = Thread.new do
       begin
-        Timeout::timeout(2) do
+        Timeout::timeout(10) do
           @in.tail(110) do |l|
             lines << l
           end
@@ -340,8 +345,10 @@ class FileTailTest < Test::Unit::TestCase
       end
     end
     appender = Thread.new do
-      until logger.stop?
-        sleep 0.1
+      Timeout::timeout(10) do
+        until lines.size == 100
+          sleep 0.1
+        end
       end
       @out.truncate 0
       @out.close
